/*
 * Copyright (c) 2013 J. Lewis Muir <jlmuir@imca-cat.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package org.imca_cat.procrun;

import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Arrays;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.imca_cat.procrun.DiscardingInputStreamConsumer;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * Tests {@link DiscardingInputStreamConsumer}.
 */
public class DiscardingInputStreamConsumerTest extends ProcrunTest {
  private ExecutorService executor;

  @Before
  public void setUp() {
    executor = Executors.newCachedThreadPool();
  }

  @After
  public void tearDown() {
    if (executor != null) executor.shutdownNow();
  }

  @Test
  public void testNoInput() throws ExecutionException, InterruptedException {
    InputStream stream = new ByteArrayInputStream(new byte[0]);
    DiscardingInputStreamConsumer consumer = new DiscardingInputStreamConsumer(stream, 128);
    byte[] expected = new byte[0];
    byte[] actual = byteBufferToArray(executor.submit(consumer).get());
    assertTrue(Arrays.equals(expected, actual));
  }

  @Test
  public void testWithInput() throws ExecutionException, InterruptedException {
    InputStream stream = toInputStream("abc");
    DiscardingInputStreamConsumer consumer = new DiscardingInputStreamConsumer(stream, 128);
    byte[] expected = new byte[0];
    byte[] actual = byteBufferToArray(executor.submit(consumer).get());
    assertTrue(Arrays.equals(expected, actual));
  }

  @Test
  public void testInputBiggerThanBuffer() throws ExecutionException, InterruptedException {
    InputStream stream = toInputStream("These aren't the droids you're looking for.");
    DiscardingInputStreamConsumer consumer = new DiscardingInputStreamConsumer(stream, 8);
    byte[] expected = new byte[0];
    byte[] actual = byteBufferToArray(executor.submit(consumer).get());
    assertTrue(Arrays.equals(expected, actual));
  }
}
